@@ -33,7 +33,8 @@ from social_auth.backends.exceptions import StopPipeline, AuthException, \
                                             AuthUnknownError, AuthTokenError, \
                                             AuthMissingParameter, \
                                             AuthStateMissing, \
-                                            AuthStateForbidden
+                                            AuthStateForbidden, \
+                                            NotAllowedToDisconnect
 from social_auth.backends.utils import build_consumer_oauth_request
 
 
@@ -81,6 +82,7 @@ class SocialAuthBackend(object):
     """A django.contrib.auth backend that authenticates the user based on
     a authentication provider response"""
     name = ''  # provider name, it's stored in database
+    supports_inactive_user = False
 
     def authenticate(self, *args, **kwargs):
         """Authenticate user using social credentials
@@ -205,10 +207,11 @@ class OAuthBackend(SocialAuthBackend):
     access_token is always stored.
     """
     EXTRA_DATA = None
+    ID_KEY = 'id'
 
     def get_user_id(self, details, response):
         """OAuth providers return an unique user id in response"""
-        return response['id']
+        return response[self.ID_KEY]
 
     def extra_data(self, user, uid, response, details):
         """Return access_token and extra defined names to store in
@@ -407,12 +410,17 @@ class BaseAuth(object):
         """Deletes current backend from user if associated.
         Override if extra operations are needed.
         """
-        if association_id:
-            UserSocialAuth.get_social_auth_for_user(user)\
-                            .get(id=association_id).delete()
+        name = self.AUTH_BACKEND.name
+        if UserSocialAuth.allowed_to_disconnect(user, name, association_id):
+            if association_id:
+                UserSocialAuth.get_social_auth_for_user(user)\
+                                .get(id=association_id).delete()
+            else:
+                UserSocialAuth.get_social_auth_for_user(user)\
+                                .filter(provider=name)\
+                                .delete()
         else:
-            UserSocialAuth.get_social_auth_for_user(user)\
-                            .filter(provider=self.AUTH_BACKEND.name).delete()
+            raise NotAllowedToDisconnect()
 
     def build_absolute_uri(self, path=None):
         """Build absolute URI for given path. Replace http:// schema with
@@ -530,11 +538,14 @@ class OpenIdAuth(BaseAuth):
 
     def openid_request(self, extra_params=None):
         """Return openid request"""
-        try:
-            return self.consumer().begin(url_add_parameters(self.openid_url(),
-                                                            extra_params))
-        except DiscoveryFailure, err:
-            raise AuthException(self, 'OpenID discovery error: %s' % err)
+        if not hasattr(self, '_openid_request'):
+            try:
+                self._openid_request = self.consumer().begin(
+                    url_add_parameters(self.openid_url(), extra_params)
+                )
+            except DiscoveryFailure, err:
+                raise AuthException(self, 'OpenID discovery error: %s' % err)
+        return self._openid_request
 
     def openid_url(self):
         """Return service provider URL.
@@ -549,6 +560,10 @@ class BaseOAuth(BaseAuth):
     """OAuth base class"""
     SETTINGS_KEY_NAME = ''
     SETTINGS_SECRET_NAME = ''
+    SCOPE_VAR_NAME = None
+    SCOPE_PARAMETER_NAME = 'scope'
+    DEFAULT_SCOPE = None
+    SCOPE_SEPARATOR = ' '
 
     def __init__(self, request, redirect):
         """Init method"""
@@ -569,6 +584,24 @@ class BaseOAuth(BaseAuth):
         return setting(cls.SETTINGS_KEY_NAME) and \
                setting(cls.SETTINGS_SECRET_NAME)
 
+    def get_scope(self):
+        """Return list with needed access scope"""
+        scope = self.DEFAULT_SCOPE or []
+        if self.SCOPE_VAR_NAME:
+            scope = scope + setting(self.SCOPE_VAR_NAME, [])
+        return scope
+
+    def get_scope_argument(self):
+        param = {}
+        scope = self.get_scope()
+        if scope:
+            param[self.SCOPE_PARAMETER_NAME] = self.SCOPE_SEPARATOR.join(scope)
+        return param
+
+    def user_data(self, access_token, *args, **kwargs):
+        """Loads user data from service. Implement in subclass"""
+        return {}
+
 
 class ConsumerBasedOAuth(BaseOAuth):
     """Consumer based mechanism OAuth authentication, fill the needed
@@ -577,12 +610,10 @@ class ConsumerBasedOAuth(BaseOAuth):
         AUTHORIZATION_URL       Authorization service url
         REQUEST_TOKEN_URL       Request token URL
         ACCESS_TOKEN_URL        Access token URL
-        SERVER_URL              Authorization server URL
     """
     AUTHORIZATION_URL = ''
     REQUEST_TOKEN_URL = ''
     ACCESS_TOKEN_URL = ''
-    SERVER_URL = ''
 
     def auth_url(self):
         """Return redirect url"""
@@ -596,11 +627,11 @@ class ConsumerBasedOAuth(BaseOAuth):
         name = self.AUTH_BACKEND.name + 'unauthorized_token_name'
         unauthed_token = self.request.session.get(name)
         if not unauthed_token:
-            raise AuthTokenError('Missing unauthorized token')
+            raise AuthTokenError(self, 'Missing unauthorized token')
 
         token = Token.from_string(unauthed_token)
         if token.key != self.data.get('oauth_token', 'no-token'):
-            raise AuthTokenError('Incorrect tokens')
+            raise AuthTokenError(self, 'Incorrect tokens')
 
         try:
             access_token = self.access_token(token)
@@ -609,7 +640,10 @@ class ConsumerBasedOAuth(BaseOAuth):
                 raise AuthCanceled(self)
             else:
                 raise
+        return self.do_auth(access_token, *args, **kwargs)
 
+    def do_auth(self, access_token, *args, **kwargs):
+        """Finish the auth process once the access_token was retrieved"""
         data = self.user_data(access_token)
         if data is not None:
             data['access_token'] = access_token.to_string()
@@ -630,10 +664,14 @@ class ConsumerBasedOAuth(BaseOAuth):
 
     def oauth_authorization_request(self, token):
         """Generate OAuth request to authorize token."""
-        return OAuthRequest.from_token_and_callback(token=token,
-                                        callback=self.redirect_uri,
-                                        http_url=self.AUTHORIZATION_URL,
-                                        parameters=self.auth_extra_arguments())
+        params = self.auth_extra_arguments() or {}
+        params.update(self.get_scope_argument())
+        return OAuthRequest.from_token_and_callback(
+            token=token,
+            callback=self.redirect_uri,
+            http_url=self.AUTHORIZATION_URL,
+            parameters=params
+        )
 
     def oauth_request(self, token, url, extra_params=None):
         """Generate OAuth request, setups callback url"""
@@ -652,10 +690,6 @@ class ConsumerBasedOAuth(BaseOAuth):
         request = self.oauth_request(token, self.ACCESS_TOKEN_URL)
         return Token.from_string(self.fetch_response(request))
 
-    def user_data(self, access_token, *args, **kwargs):
-        """Loads user data from service"""
-        raise NotImplementedError('Implement in subclass')
-
     @property
     def consumer(self):
         """Setups consumer"""
@@ -674,54 +708,60 @@ class BaseOAuth2(BaseOAuth):
     """
     AUTHORIZATION_URL = None
     ACCESS_TOKEN_URL = None
-    SCOPE_SEPARATOR = ' '
     RESPONSE_TYPE = 'code'
-    SCOPE_VAR_NAME = None
-    DEFAULT_SCOPE = None
     REDIRECT_STATE = True
+    STATE_PARAMETER = True
 
     def state_token(self):
         """Generate csrf token to include as state parameter."""
         return get_random_string(32)
 
-    def get_redirect_uri(self, state):
+    def get_redirect_uri(self, state=None):
         """Build redirect_uri with redirect_state parameter."""
         uri = self.redirect_uri
-        if self.REDIRECT_STATE:
+        if self.REDIRECT_STATE and state:
             uri = url_add_parameters(uri, {'redirect_state': state})
         return uri
 
-    def auth_url(self):
-        """Return redirect url"""
+    def auth_params(self, state=None):
         client_id, client_secret = self.get_key_and_secret()
-        state = self.state_token()
-        # Store state in session for further request validation. The state
-        # value is passed as state parameter (as specified in OAuth2 spec), but
-        # also added to redirect_uri, that way we can still verify the request
-        # if the provider doesn't implement the state parameter.
-        self.request.session[self.AUTH_BACKEND.name + '_state'] = state
-        args = {
+        params = {
             'client_id': client_id,
-            'state': state,
             'redirect_uri': self.get_redirect_uri(state)
         }
-
-        scope = self.get_scope()
-        if scope:
-            args['scope'] = self.SCOPE_SEPARATOR.join(self.get_scope())
+        if self.STATE_PARAMETER and state:
+            params['state'] = state
         if self.RESPONSE_TYPE:
-            args['response_type'] = self.RESPONSE_TYPE
+            params['response_type'] = self.RESPONSE_TYPE
+        return params
+
+    def auth_url(self):
+        """Return redirect url"""
+        if self.STATE_PARAMETER or self.REDIRECT_STATE:
+            # Store state in session for further request validation. The state
+            # value is passed as state parameter (as specified in OAuth2 spec),
+            # but also added to redirect_uri, that way we can still verify the
+            # request if the provider doesn't implement the state parameter.
+            state = self.state_token()
+            self.request.session[self.AUTH_BACKEND.name + '_state'] = state
+        else:
+            state = None
+
+        params = self.auth_params(state)
+        params.update(self.get_scope_argument())
+        params.update(self.auth_extra_arguments())
 
-        args.update(self.auth_extra_arguments())
         if self.request.META.get('QUERY_STRING'):
             query_string = '&' + self.request.META['QUERY_STRING']
         else:
             query_string = ''
-        return self.AUTHORIZATION_URL + '?' + urlencode(args) + query_string
+        return self.AUTHORIZATION_URL + '?' + urlencode(params) + query_string
 
     def validate_state(self):
         """Validate state value. Raises exception on error, returns state
         value if valid."""
+        if not self.STATE_PARAMETER and not self.REDIRECT_STATE:
+            return None
         state = self.request.session.get(self.AUTH_BACKEND.name + '_state')
         request_state = self.data.get('state') or \
                         self.data.get('redirect_state')
@@ -733,25 +773,31 @@ class BaseOAuth2(BaseOAuth):
             raise AuthStateForbidden(self)
         return state
 
-    def auth_complete(self, *args, **kwargs):
-        """Completes loging process, must return user instance"""
-        if self.data.get('error'):
+    def process_error(self, data):
+        if data.get('error'):
             error = self.data.get('error_description') or self.data['error']
             raise AuthFailed(self, error)
 
-        state = self.validate_state()
+    def auth_complete_params(self, state=None):
         client_id, client_secret = self.get_key_and_secret()
-        params = {
+        return {
             'grant_type': 'authorization_code',  # request auth code
             'code': self.data.get('code', ''),  # server response code
             'client_id': client_id,
             'client_secret': client_secret,
             'redirect_uri': self.get_redirect_uri(state)
         }
-        headers = {'Content-Type': 'application/x-www-form-urlencoded',
-                    'Accept': 'application/json'}
+
+    def auth_complete_headers(self):
+        return {'Content-Type': 'application/x-www-form-urlencoded',
+                'Accept': 'application/json'}
+
+    def auth_complete(self, *args, **kwargs):
+        """Completes loging process, must return user instance"""
+        self.process_error(self.data)
+        params = self.auth_complete_params(self.validate_state())
         request = Request(self.ACCESS_TOKEN_URL, data=urlencode(params),
-                          headers=headers)
+                          headers=self.auth_complete_headers())
 
         try:
             response = simplejson.loads(dsa_urlopen(request).read())
@@ -763,25 +809,20 @@ class BaseOAuth2(BaseOAuth):
         except (ValueError, KeyError):
             raise AuthUnknownError(self)
 
-        if response.get('error'):
-            error = response.get('error_description') or response.get('error')
-            raise AuthFailed(self, error)
-        else:
-            data = self.user_data(response['access_token'], response)
-            response.update(data or {})
-            kwargs.update({
-                'auth': self,
-                'response': response,
-                self.AUTH_BACKEND.name: True
-            })
-            return authenticate(*args, **kwargs)
+        self.process_error(response)
+        return self.do_auth(response['access_token'], response=response)
 
-    def get_scope(self):
-        """Return list with needed access scope"""
-        scope = self.DEFAULT_SCOPE or []
-        if self.SCOPE_VAR_NAME:
-            scope = scope + setting(self.SCOPE_VAR_NAME, [])
-        return scope
+    def do_auth(self, access_token, *args, **kwargs):
+        """Finish the auth process once the access_token was retrieved"""
+        data = self.user_data(access_token, *args, **kwargs)
+        response = kwargs.get('response') or {}
+        response.update(data or {})
+        kwargs.update({
+            'auth': self,
+            'response': response,
+            self.AUTH_BACKEND.name: True
+        })
+        return authenticate(*args, **kwargs)
 
 
 # Backend loading was previously performed via the
