@@ -6,6 +6,11 @@ class SocialAuthBaseException(ValueError):
     pass
 
 
+class NotAllowedToDisconnect(SocialAuthBaseException):
+    """User is not allowed to disconnect it's social account."""
+    pass
+
+
 class StopPipeline(SocialAuthBaseException):
     """Stop pipeline process exception.
     Raise this exception to stop the rest of the pipeline process.
